@@ -1,17 +1,13 @@
-"""
-    test_intl
-    ~~~~~~~~~
+"""Test message patching for internationalization purposes.
 
-    Test message patching for internationalization purposes.  Runs the text
-    builder in the test root.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
+Runs the text builder in the test root.
 """
 
 import os
 import re
 
+import docutils
+import pygments
 import pytest
 from babel.messages import mofile, pofile
 from babel.messages.catalog import Catalog
@@ -20,7 +16,6 @@ from docutils import nodes
 from sphinx import locale
 from sphinx.testing.util import (assert_node, assert_not_re_search, assert_re_search,
                                  assert_startswith, etree_parse, path, strip_escseq)
-from sphinx.util import docutils
 
 sphinx_intl = pytest.mark.sphinx(
     testroot='intl',
@@ -30,9 +25,11 @@ sphinx_intl = pytest.mark.sphinx(
     },
 )
 
+pygments_version = tuple(int(v) for v in pygments.__version__.split('.'))
+
 
 def read_po(pathname):
-    with pathname.open() as f:
+    with pathname.open(encoding='utf-8') as f:
         return pofile.read_po(f)
 
 
@@ -44,7 +41,7 @@ def write_mo(pathname, po):
 @pytest.fixture(autouse=True)
 def setup_intl(app_params):
     srcdir = path(app_params.kwargs['srcdir'])
-    for dirpath, dirs, files in os.walk(srcdir):
+    for dirpath, _dirs, files in os.walk(srcdir):
         dirpath = path(dirpath)
         for f in [f for f in files if f.endswith('.po')]:
             po = dirpath / f
@@ -108,7 +105,7 @@ def test_text_emit_warnings(app, warning):
 def test_text_warning_node(app):
     app.build()
     # test warnings in translation
-    result = (app.outdir / 'warnings.txt').read_text()
+    result = (app.outdir / 'warnings.txt').read_text(encoding='utf8')
     expect = ("3. I18N WITH REST WARNINGS"
               "\n**************************\n"
               "\nLINE OF >>``<<BROKEN LITERAL MARKUP.\n")
@@ -122,7 +119,7 @@ def test_text_warning_node(app):
 def test_text_title_underline(app):
     app.build()
     # --- simple translation; check title underlines
-    result = (app.outdir / 'bom.txt').read_text()
+    result = (app.outdir / 'bom.txt').read_text(encoding='utf8')
     expect = ("2. Datei mit UTF-8"
               "\n******************\n"  # underline matches new translation
               "\nThis file has umlauts: äöü.\n")
@@ -135,7 +132,7 @@ def test_text_title_underline(app):
 def test_text_subdirs(app):
     app.build()
     # --- check translation in subdirs
-    result = (app.outdir / 'subdir' / 'index.txt').read_text()
+    result = (app.outdir / 'subdir' / 'index.txt').read_text(encoding='utf8')
     assert_startswith(result, "1. subdir contents\n******************\n")
 
 
@@ -145,7 +142,7 @@ def test_text_subdirs(app):
 def test_text_inconsistency_warnings(app, warning):
     app.build()
     # --- check warnings for inconsistency in number of references
-    result = (app.outdir / 'refs_inconsistency.txt').read_text()
+    result = (app.outdir / 'refs_inconsistency.txt').read_text(encoding='utf8')
     expect = ("8. I18N WITH REFS INCONSISTENCY"
               "\n*******************************\n"
               "\n* FOR CITATION [ref3].\n"
@@ -189,13 +186,39 @@ def test_text_inconsistency_warnings(app, warning):
     assert_re_search(expected_citation_warning_expr, warnings)
 
 
+@sphinx_intl
+@pytest.mark.sphinx('text')
+@pytest.mark.test_params(shared_result='test_intl_basic')
+def test_noqa(app, warning):
+    app.build()
+    result = (app.outdir / 'noqa.txt').read_text(encoding='utf8')
+    expect = r"""FIRST SECTION
+*************
+
+TRANSLATED TEXT WITHOUT REFERENCE.
+
+TEST noqa WHITESPACE INSENSITIVITY.
+
+"#noqa" IS ESCAPED AT THE END OF THIS STRING. #noqa
+
+
+NEXT SECTION WITH PARAGRAPH TO TEST BARE noqa
+*********************************************
+
+Some text, again referring to the section: NEXT SECTION WITH PARAGRAPH
+TO TEST BARE noqa.
+"""
+    assert result == expect
+    assert "next-section" not in getwarning(warning)
+
+
 @sphinx_intl
 @pytest.mark.sphinx('text')
 @pytest.mark.test_params(shared_result='test_intl_basic')
 def test_text_literalblock_warnings(app, warning):
     app.build()
     # --- check warning for literal block
-    result = (app.outdir / 'literalblock.txt').read_text()
+    result = (app.outdir / 'literalblock.txt').read_text(encoding='utf8')
     expect = ("9. I18N WITH LITERAL BLOCK"
               "\n**************************\n"
               "\nCORRECT LITERAL BLOCK:\n"
@@ -217,7 +240,7 @@ def test_text_literalblock_warnings(app, warning):
 def test_text_definition_terms(app):
     app.build()
     # --- definition terms: regression test for #975, #2198, #2205
-    result = (app.outdir / 'definition_terms.txt').read_text()
+    result = (app.outdir / 'definition_terms.txt').read_text(encoding='utf8')
     expect = ("13. I18N WITH DEFINITION TERMS"
               "\n******************************\n"
               "\nSOME TERM"
@@ -237,14 +260,30 @@ def test_text_definition_terms(app):
 def test_text_glossary_term(app, warning):
     app.build()
     # --- glossary terms: regression test for #1090
-    result = (app.outdir / 'glossary_terms.txt').read_text()
-    expect = ("18. I18N WITH GLOSSARY TERMS"
-              "\n****************************\n"
-              "\nSOME NEW TERM"
-              "\n   THE CORRESPONDING GLOSSARY\n"
-              "\nSOME OTHER NEW TERM"
-              "\n   THE CORRESPONDING GLOSSARY #2\n"
-              "\nLINK TO *SOME NEW TERM*.\n")
+    result = (app.outdir / 'glossary_terms.txt').read_text(encoding='utf8')
+    expect = (r"""18. I18N WITH GLOSSARY TERMS
+****************************
+
+SOME NEW TERM
+   THE CORRESPONDING GLOSSARY
+
+SOME OTHER NEW TERM
+   THE CORRESPONDING GLOSSARY #2
+
+LINK TO *SOME NEW TERM*.
+
+TRANSLATED GLOSSARY SHOULD BE SORTED BY TRANSLATED TERMS:
+
+TRANSLATED TERM XXX
+   DEFINE XXX
+
+TRANSLATED TERM YYY
+   DEFINE YYY
+
+TRANSLATED TERM ZZZ
+VVV
+   DEFINE ZZZ
+""")
     assert result == expect
     warnings = getwarning(warning)
     assert 'term not in glossary' not in warnings
@@ -256,7 +295,7 @@ def test_text_glossary_term(app, warning):
 def test_text_glossary_term_inconsistencies(app, warning):
     app.build()
     # --- glossary term inconsistencies: regression test for #1090
-    result = (app.outdir / 'glossary_terms_inconsistency.txt').read_text()
+    result = (app.outdir / 'glossary_terms_inconsistency.txt').read_text(encoding='utf8')
     expect = ("19. I18N WITH GLOSSARY TERMS INCONSISTENCY"
               "\n******************************************\n"
               "\n1. LINK TO *SOME NEW TERM*.\n")
@@ -289,7 +328,7 @@ def test_gettext_section(app):
 def test_text_section(app):
     app.build()
     # --- section
-    result = (app.outdir / 'section.txt').read_text()
+    result = (app.outdir / 'section.txt').read_text(encoding='utf8')
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'section.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -301,7 +340,7 @@ def test_text_section(app):
 def test_text_seealso(app):
     app.build()
     # --- seealso
-    result = (app.outdir / 'seealso.txt').read_text()
+    result = (app.outdir / 'seealso.txt').read_text(encoding='utf8')
     expect = ("12. I18N WITH SEEALSO"
               "\n*********************\n"
               "\nSee also: SHORT TEXT 1\n"
@@ -318,7 +357,7 @@ def test_text_seealso(app):
 def test_text_figure_captions(app):
     app.build()
     # --- figure captions: regression test for #940
-    result = (app.outdir / 'figure.txt').read_text()
+    result = (app.outdir / 'figure.txt').read_text(encoding='utf8')
     expect = ("14. I18N WITH FIGURE CAPTION"
               "\n****************************\n"
               "\n   [image]MY CAPTION OF THE FIGURE\n"
@@ -362,7 +401,7 @@ def test_text_figure_captions(app):
 def test_text_rubric(app):
     app.build()
     # --- rubric: regression test for pull request #190
-    result = (app.outdir / 'rubric.txt').read_text()
+    result = (app.outdir / 'rubric.txt').read_text(encoding='utf8')
     expect = ("I18N WITH RUBRIC"
               "\n****************\n"
               "\n-[ RUBRIC TITLE ]-\n"
@@ -380,7 +419,7 @@ def test_text_rubric(app):
 def test_text_docfields(app):
     app.build()
     # --- docfields
-    result = (app.outdir / 'docfields.txt').read_text()
+    result = (app.outdir / 'docfields.txt').read_text(encoding='utf8')
     expect = ("21. I18N WITH DOCFIELDS"
               "\n***********************\n"
               "\nclass Cls1\n"
@@ -411,7 +450,7 @@ def test_text_admonitions(app):
     # --- admonitions
     # #1206: gettext did not translate admonition directive's title
     # seealso: https://docutils.sourceforge.io/docs/ref/rst/directives.html#admonitions
-    result = (app.outdir / 'admonitions.txt').read_text()
+    result = (app.outdir / 'admonitions.txt').read_text(encoding='utf8')
     directives = (
         "attention", "caution", "danger", "error", "hint",
         "important", "note", "tip", "warning", "admonition")
@@ -458,7 +497,7 @@ def test_gettext_table(app):
 def test_text_table(app):
     app.build()
     # --- toctree
-    result = (app.outdir / 'table.txt').read_text()
+    result = (app.outdir / 'table.txt').read_text(encoding='utf8')
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'table.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -471,11 +510,11 @@ def test_text_toctree(app):
     app.build()
     # --- toctree (index.rst)
     # Note: index.rst contains contents that is not shown in text.
-    result = (app.outdir / 'index.txt').read_text()
+    result = (app.outdir / 'index.txt').read_text(encoding='utf8')
     assert 'CONTENTS' in result
     assert 'TABLE OF CONTENTS' in result
     # --- toctree (toctree.rst)
-    result = (app.outdir / 'toctree.txt').read_text()
+    result = (app.outdir / 'toctree.txt').read_text(encoding='utf8')
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'toctree.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -499,7 +538,7 @@ def test_gettext_topic(app):
 def test_text_topic(app):
     app.build()
     # --- topic
-    result = (app.outdir / 'topic.txt').read_text()
+    result = (app.outdir / 'topic.txt').read_text(encoding='utf8')
     expect = read_po(app.srcdir / 'xx' / 'LC_MESSAGES' / 'topic.po')
     for expect_msg in [m for m in expect if m.id]:
         assert expect_msg.string in result
@@ -617,7 +656,7 @@ def test_gettext_dont_rebuild_mo(make_app, app_params):
 def test_html_meta(app):
     app.build()
     # --- test for meta
-    result = (app.outdir / 'index.html').read_text()
+    result = (app.outdir / 'index.html').read_text(encoding='utf8')
     expected_expr = '<meta content="TESTDATA FOR I18N" name="description" />'
     assert expected_expr in result
     expected_expr = '<meta content="I18N, SPHINX, MARKUP" name="keywords" />'
@@ -633,7 +672,7 @@ def test_html_footnotes(app):
     app.build()
     # --- test for #955 cant-build-html-with-footnotes-when-using
     # expect no error by build
-    (app.outdir / 'footnote.html').read_text()
+    (app.outdir / 'footnote.html').read_text(encoding='utf8')
 
 
 @sphinx_intl
@@ -642,7 +681,7 @@ def test_html_footnotes(app):
 def test_html_undefined_refs(app):
     app.build()
     # --- links to undefined reference
-    result = (app.outdir / 'refs_inconsistency.html').read_text()
+    result = (app.outdir / 'refs_inconsistency.html').read_text(encoding='utf8')
 
     expected_expr = ('<a class="reference external" '
                      'href="http://www.example.com">reference</a>')
@@ -664,7 +703,7 @@ def test_html_undefined_refs(app):
 def test_html_index_entries(app):
     app.build()
     # --- index entries: regression test for #976
-    result = (app.outdir / 'genindex.html').read_text()
+    result = (app.outdir / 'genindex.html').read_text(encoding='utf8')
 
     def wrap(tag, keyword):
         start_tag = "<%s[^>]*>" % tag
@@ -702,7 +741,7 @@ def test_html_index_entries(app):
 def test_html_versionchanges(app):
     app.build()
     # --- versionchanges
-    result = (app.outdir / 'versionchange.html').read_text()
+    result = (app.outdir / 'versionchange.html').read_text(encoding='utf8')
 
     def get_content(result, name):
         matched = re.search(r'<div class="%s">\n*(.*?)</div>' % name,
@@ -739,7 +778,7 @@ def test_html_docfields(app):
     app.build()
     # --- docfields
     # expect no error by build
-    (app.outdir / 'docfields.html').read_text()
+    (app.outdir / 'docfields.html').read_text(encoding='utf8')
 
 
 @sphinx_intl
@@ -748,7 +787,7 @@ def test_html_docfields(app):
 def test_html_template(app):
     app.build()
     # --- gettext template
-    result = (app.outdir / 'contents.html').read_text()
+    result = (app.outdir / 'contents.html').read_text(encoding='utf8')
     assert "WELCOME" in result
     assert "SPHINX 2013.120" in result
 
@@ -1045,7 +1084,7 @@ def test_xml_label_targets(app):
 def test_additional_targets_should_not_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').read_text()
+    result = (app.outdir / 'literalblock.html').read_text(encoding='utf8')
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -1060,8 +1099,13 @@ def test_additional_targets_should_not_be_translated(app):
     assert_count(expected_expr, result, 1)
 
     # C code block with lang should not be translated but be *C* highlighted
-    expected_expr = ("""<span class="cp">#include</span> """
-                     """<span class="cpf">&lt;stdio.h&gt;</span>""")
+    if pygments_version < (2, 10, 0):
+        expected_expr = ("""<span class="cp">#include</span> """
+                         """<span class="cpf">&lt;stdio.h&gt;</span>""")
+    else:
+        expected_expr = ("""<span class="cp">#include</span>"""
+                         """<span class="w"> </span>"""
+                         """<span class="cpf">&lt;stdio.h&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
     # literal block in list item should not be translated
@@ -1081,7 +1125,7 @@ def test_additional_targets_should_not_be_translated(app):
 
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').read_text()
+    result = (app.outdir / 'raw.html').read_text(encoding='utf8')
 
     # raw block should not be translated
     if docutils.__version_info__ < (0, 17):
@@ -1093,7 +1137,7 @@ def test_additional_targets_should_not_be_translated(app):
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').read_text()
+    result = (app.outdir / 'figure.html').read_text(encoding='utf8')
 
     # src for image block should not be translated (alt is translated)
     expected_expr = """<img alt="I18N -&gt; IMG" src="_images/i18n.png" />"""
@@ -1123,7 +1167,7 @@ def test_additional_targets_should_not_be_translated(app):
 def test_additional_targets_should_be_translated(app):
     app.build()
     # [literalblock.txt]
-    result = (app.outdir / 'literalblock.html').read_text()
+    result = (app.outdir / 'literalblock.html').read_text(encoding='utf8')
 
     # title should be translated
     expected_expr = 'CODE-BLOCKS'
@@ -1138,8 +1182,13 @@ def test_additional_targets_should_be_translated(app):
     assert_count(expected_expr, result, 1)
 
     # C code block with lang should be translated and be *C* highlighted
-    expected_expr = ("""<span class="cp">#include</span> """
-                     """<span class="cpf">&lt;STDIO.H&gt;</span>""")
+    if pygments_version < (2, 10, 0):
+        expected_expr = ("""<span class="cp">#include</span> """
+                         """<span class="cpf">&lt;STDIO.H&gt;</span>""")
+    else:
+        expected_expr = ("""<span class="cp">#include</span>"""
+                         """<span class="w"> </span>"""
+                         """<span class="cpf">&lt;STDIO.H&gt;</span>""")
     assert_count(expected_expr, result, 1)
 
     # literal block in list item should be translated
@@ -1157,9 +1206,12 @@ def test_additional_targets_should_be_translated(app):
         """<span class="c1"># SYS IMPORTING</span>""")
     assert_count(expected_expr, result, 1)
 
+    # '#noqa' should remain in literal blocks.
+    assert_count("#noqa", result, 1)
+
     # [raw.txt]
 
-    result = (app.outdir / 'raw.html').read_text()
+    result = (app.outdir / 'raw.html').read_text(encoding='utf8')
 
     # raw block should be translated
     if docutils.__version_info__ < (0, 17):
@@ -1171,7 +1223,7 @@ def test_additional_targets_should_be_translated(app):
 
     # [figure.txt]
 
-    result = (app.outdir / 'figure.html').read_text()
+    result = (app.outdir / 'figure.html').read_text(encoding='utf8')
 
     # alt and src for image block should be translated
     expected_expr = """<img alt="I18N -&gt; IMG" src="_images/img.png" />"""
@@ -1288,6 +1340,44 @@ def getwarning(warnings):
     return strip_escseq(warnings.getvalue().replace(os.sep, '/'))
 
 
+@pytest.mark.sphinx('html', testroot='basic',
+                    srcdir='gettext_allow_fuzzy_translations',
+                    confoverrides={
+                        'language': 'de',
+                        'gettext_allow_fuzzy_translations': True
+                    })
+def test_gettext_allow_fuzzy_translations(app):
+    locale_dir = app.srcdir / 'locales' / 'de' / 'LC_MESSAGES'
+    locale_dir.makedirs()
+    with (locale_dir / 'index.po').open('wb') as f:
+        catalog = Catalog()
+        catalog.add('features', 'FEATURES', flags=('fuzzy',))
+        pofile.write_po(f, catalog)
+
+    app.build()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert 'FEATURES' in content
+
+
+@pytest.mark.sphinx('html', testroot='basic',
+                    srcdir='gettext_disallow_fuzzy_translations',
+                    confoverrides={
+                        'language': 'de',
+                        'gettext_allow_fuzzy_translations': False
+                    })
+def test_gettext_disallow_fuzzy_translations(app):
+    locale_dir = app.srcdir / 'locales' / 'de' / 'LC_MESSAGES'
+    locale_dir.makedirs()
+    with (locale_dir / 'index.po').open('wb') as f:
+        catalog = Catalog()
+        catalog.add('features', 'FEATURES', flags=('fuzzy',))
+        pofile.write_po(f, catalog)
+
+    app.build()
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert 'FEATURES' not in content
+
+
 @pytest.mark.sphinx('html', testroot='basic', confoverrides={'language': 'de'})
 def test_customize_system_message(make_app, app_params, sphinx_test_tempdir):
     try:
@@ -1309,7 +1399,7 @@ def test_customize_system_message(make_app, app_params, sphinx_test_tempdir):
         assert app.translator.gettext('Quick search') == 'QUICK SEARCH'
 
         app.build()
-        content = (app.outdir / 'index.html').read_text()
+        content = (app.outdir / 'index.html').read_text(encoding='utf8')
         assert 'QUICK SEARCH' in content
     finally:
         locale.translators.clear()
